/* Copyright(c) 2005 arton
 *
 * Usage of the works is permitted provided that this instrument is retained with the works,
 * so that any entity that uses the works is notified of this instrument.
 *
 * DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
 *
 * SimpleContainer.java
 *
 * Created on 2005/09/14, 3:39
 *
 */

package jp.co.gihyo.book.di.ch2.di;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * \[Xoh𗘗pPDIReiB
 *
 * @author arton
 */
public class SimpleContainer {
    
    static final Pattern DEF_OBJECT = Pattern.compile("^class\\.(.+)$");
    
    Map<Class, Object> interfaces;
    Map<String, Object> objects;
    /** 
     * w肵\[X𗘗pCX^X𐶐B
     * @param resource \[X
     */
    public SimpleContainer(String resource) throws Exception {
        objects = new HashMap<String, Object>();
        interfaces = new HashMap<Class, Object>();

        ResourceBundle rb = ResourceBundle.getBundle(resource);
        Map<String, String> resources = new HashMap<String, String>();
        for (Enumeration<String> keys = rb.getKeys(); keys.hasMoreElements();) {
            String key = keys.nextElement();
            Matcher m = DEF_OBJECT.matcher(key);
            if (m.matches()) {
                Class declClass = Class.forName(rb.getString(key));
                Object o = declClass.newInstance();
                objects.put(m.group(1), o);
                for(Class c : declClass.getInterfaces()) {
                    if (!c.equals(java.io.Serializable.class)) {
                        interfaces.put(c, o);
                    }
                }
            } else {
                resources.put(key, rb.getString(key));
            }
        }
        for (Map.Entry<String, Object> e : objects.entrySet()) {
            setupObject(e.getKey(), e.getValue(), resources);
            for (Map.Entry<Class, Object> ent : interfaces.entrySet()) {
                if (ent.getValue() != e.getValue()) {
                    setupObject(ent.getKey(), e.getValue(), ent.getValue());
                }
            }
        }
    }
    
    void setupObject(String name, Object o, Map<String, String> resources) {
        Class c = o.getClass();
        Pattern p = Pattern.compile("^" + name + "\\.(.+)$");
        for (Map.Entry<String, String> e : resources.entrySet()) {
            Matcher m = p.matcher(e.getKey());
            if (m.matches()) {
                setProperty(c, o, m.group(1), e.getValue());
            }
        }
    }
    
    void setupObject(Class c, Object o, Object value) {
        try {
            for (Method m : o.getClass().getMethods()) {
                Class<?>[] cs = m.getParameterTypes();
                if (cs.length == 1 && cs[0].isAssignableFrom(c)) {
                    m.invoke(o, value);
                    break;
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.getCause().printStackTrace();
        }
    }

    private void setProperty(Class c, Object o, String prop, String value) {
        try {
            for (Method m : c.getMethods()) {
                Object dependency;
                Class<?>[] cs = m.getParameterTypes();
                if (cs.length == 1 && m.getName().equals(setterName(prop))) {
                    m.invoke(o, coerce(value, cs[0]));
                    return;
                }
            }
            System.err.println(setterName(prop) + " not found for " + c);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.getCause().printStackTrace();
        }
    }
    
    Object coerce(String value, Class<?> c) {
        if (c.isAssignableFrom(String.class)) {
            return value;
        } else if (c.isAssignableFrom(Integer.TYPE) || c.isAssignableFrom(Integer.class)) {
            return Integer.valueOf(value);
        } else if (c.isAssignableFrom(Boolean.TYPE) || c.isAssignableFrom(Boolean.class)) {
            return Boolean.valueOf(value);
        }
        throw new IllegalArgumentException("bad type:" + c + ", value=" + value);
    }
    
    String setterName(String s) {
        return "set" + Character.toUpperCase(s.charAt(0)) + s.substring(1);
    }
    
    /**
     * w肵IuWFNg擾B
     * YIuWFNg`̏ꍇɂnullԑB
     * @param name \[X̖
     */
    public Object getObject(String name) {
        return objects.get(name);
    }
}
