/* Copyright(c) 2005 arton
 *
 * Usage of the works is permitted provided that this instrument is retained with the works,
 * so that any entity that uses the works is notified of this instrument.
 *
 * DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
 *
 * AnnotationInjectContainer.java
 *
 * Created on 2005/09/24, 1:45
 *
 */
package jp.co.gihyo.book.di.ch3.ano;

import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Field;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * \[Xoh𗘗pPȃAme[VttB[hCWFNVsDIReiB
 *
 * @author arton
 */
public class AnnotationInjectContainer {

    static final Pattern DEF_OBJECT = Pattern.compile("^class\\.(.+)$");
    static final Integer ZERO = new Integer(0);
    Map<String, Object> objects;
    
    /** 
     * w肵\[X𗘗pCX^X𐶐B
     * @param resource \[X
     */
    public AnnotationInjectContainer(String resource) throws Exception {
        objects = new HashMap<String, Object>();
        ResourceBundle rb = ResourceBundle.getBundle(resource);
        Map<String, String> resources = new HashMap<String, String>();
        Map<Class, Object> interfaces = new HashMap<Class, Object>();
        for (Enumeration<String> keys = rb.getKeys(); keys.hasMoreElements();) {
            String key = keys.nextElement();
            Matcher m = DEF_OBJECT.matcher(key);
            if (m.matches()) {
                Class<?> c = Class.forName(rb.getString(key));
                Object o = c.newInstance();
                Injectable injectable = c.getAnnotation(Injectable.class);
                if (injectable != null) {
                    interfaces.put(injectable.value(), o);
                }
                objects.put(m.group(1), o);
            } else {
                resources.put(key, rb.getString(key));
            }
        }
        for (Object o : objects.values()) {
            injectFields(o, interfaces, resources);
        }
    }
    
    void injectFields(Object o, Map<Class, Object>interfaces, Map<String, String>resources) 
        throws IllegalAccessException {
        for (Class c = o.getClass(); c != null && c != Object.class; c = c.getSuperclass()) {
            Field[] fs = c.getDeclaredFields();
            for (Field f : fs) {
                injectField(o, f, interfaces, resources);
            }
        }
    }
    void injectField(Object o, Field f, Map<Class, Object>interfaces, Map<String, String>resources) 
        throws IllegalAccessException {
        Inject inject = f.getAnnotation(Inject.class);
        if (inject == null) {
            return;
        }
        if (!f.isAccessible()) {
            enableAccessibility(f);
        }
        if (inject.type() == Inject.Type.INTERFACE) {
            f.set(o, interfaces.get(f.getType()));
        } else if (inject.type() == Inject.Type.NAME) {
            f.set(o, coerce(resources.get(getName(f, inject)), f.getType()));
        } else {
            assert false;
        }
    }

    String getName(Field f, Inject i) {
        assert i.value() != null;
        return (i.value().length() == 0) ? f.getName() : i.value();
    }
    
    Object coerce(String value, Class<?> c) {
        if (c.isAssignableFrom(String.class)) {
            return value;
        } else if (c.isAssignableFrom(Integer.TYPE) || c.isAssignableFrom(Integer.class)) {
            return (value == null) ? ZERO : Integer.valueOf(value);
        } else if (c.isAssignableFrom(Boolean.TYPE) || c.isAssignableFrom(Boolean.class)) {
            return (value == null) ? Boolean.FALSE : Boolean.valueOf(value);
        }
        throw new IllegalArgumentException("bad type:" + c + ", value=" + value);
    }
    
    void enableAccessibility(Field f) {
        try {
            f.setAccessible(true);
        } catch (SecurityException e) {
        }
    }

    /**
     * w肵IuWFNg擾B
     * YIuWFNg`̏ꍇɂnullԑB
     * @param name \[X̖
     */
    public Object getObject(String name) {
        return objects.get(name);
    }
}
