/* Copyright(c) 2005 arton
 *
 * Usage of the works is permitted provided that this instrument is retained with the works,
 * so that any entity that uses the works is notified of this instrument.
 *
 * DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
 *
 * ConstructorInjectContainer.java
 *
 * Created on 2005/09/20, 20:24
 *
 */

package jp.co.gihyo.book.di.ch3.ctr;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * \[Xoh𗘗pPȃRXgN^CWFNVsDIReiB
 *
 * @author arton
 */
public class ConstructorInjectContainer {

    static final Pattern DEF_OBJECT = Pattern.compile("^class\\.(.+)$");
    static final Pattern DEF_STRING = Pattern.compile("^\"(.+)\"$");
    static final Pattern DEF_NUMBER = Pattern.compile("^\\d+$");
    
    Map<String, Object> objects;
    
    class CtrParam {
        String name;
        Class cls;
        CtrParam(String key, String clsName) throws ClassNotFoundException {
            name = key;
            cls = Class.forName(clsName);
        }
        void setupArgs(Map<String, String> resources) throws Exception {
            if (objects.get(name) != null) {
                return;
            }
            StringBuilder sb = new StringBuilder(name).append(".ctr.0");
            int loc = sb.length() - 1;
            ArrayList<Object> list = new ArrayList<Object>();
            for (int i = 0;; i++) {
                sb.replace(loc, loc + 1, String.valueOf(i));
                String value = resources.get(sb.toString());
                if (value == null) {
                    break;
                }
                if (checkNumber(value, list) || checkString(value, list)) {
                    continue;
                }
                if (objects.get(value) == null) {
                    return;
                }
                list.add(objects.get(value));
            }
            Constructor[] ctrs = cls.getConstructors();
            for (int i = 0; i < ctrs.length; i++) {
                if (ctrs[i].getParameterTypes().length == list.size()) {
                    try {
                        objects.put(name, ctrs[i].newInstance(list.toArray()));
                        break;
                    } catch (IllegalArgumentException e) {
                        ; // ignore (bad argument)
                    }
                }
            }
        }
        boolean checkNumber(String v, List<Object> list) {
            Matcher m = DEF_NUMBER.matcher(v);
            if (m.matches()) {
                list.add(Integer.parseInt(v));
                return true;
            }
            return false;
        }
        boolean checkString(String v, List<Object> list) {
            Matcher m = DEF_STRING.matcher(v);
            if (m.matches()) {
                list.add(m.group(1));
                return true;
            }
            return false;
        }
    }
    /** 
     * w肵\[X𗘗pCX^X𐶐B
     * @param resource \[X
     */
    public ConstructorInjectContainer(String resource) throws Exception {
        objects = new HashMap<String, Object>();
        Map<String, String> resources = new HashMap<String, String>();
        Set<CtrParam> cls = new HashSet<CtrParam>();
        ResourceBundle rb = ResourceBundle.getBundle(resource);
        for (Enumeration<String> keys = rb.getKeys(); keys.hasMoreElements();) {
            String key = keys.nextElement();
            Matcher m = DEF_OBJECT.matcher(key);
            if (m.matches()) {
                cls.add(new CtrParam(m.group(1), rb.getString(key)));
            } else {
                resources.put(key, rb.getString(key));
            }
        }
        for (int i = 0; i < cls.size(); i++) {
            for (CtrParam ctr : cls) {
                ctr.setupArgs(resources);
            }
        }
    }
    
    /**
     * w肵IuWFNg擾B
     * YIuWFNg`̏ꍇɂnullԑB
     * @param name \[X̖
     */
    public Object getObject(String name) {
        return objects.get(name);
    }
}
