/* Copyright(c) 2005 arton
 *
 * Usage of the works is permitted provided that this instrument is retained with the works,
 * so that any entity that uses the works is notified of this instrument.
 *
 * DISCLAIMER: THE WORKS ARE WITHOUT WARRANTY.
 *
 * InterfaceInjectContainer.java
 *
 * Created on 2005/09/21, 1:32
 *
 */
package jp.co.gihyo.book.di.ch3.intf;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import static jp.co.gihyo.book.di.ch3.intf.InjectorHelper.invoke;

/**
 * \[Xoh𗘗pPȃC^[tFCXCWFNVsDIReiB
 *
 * @author arton
 */
public class InterfaceInjectContainer {
    
    static final Pattern NUMBER = Pattern.compile("^\\d+$");
    static final Pattern STRING = Pattern.compile("^\"(.*?)\"$");
    
    Map<String, Object> objects;
    
    /** 
     * w肵\[X𗘗pCX^X𐶐B
     * @param resource \[X
     */
    public InterfaceInjectContainer(String resource) throws Exception {
        objects = new HashMap<String, Object>();
        List<Injector> injectors = new ArrayList<Injector>();
        ResourceBundle rb = ResourceBundle.getBundle(resource);
        for (Enumeration<String> keys = rb.getKeys(); keys.hasMoreElements();) {
            String key = keys.nextElement();
            if (key.indexOf('.') >= 0) {
                // C^[tFCX`
                injectors.add(createInjector(Class.forName(key), rb.getString(key)));
            } else {
                // IuWFNg`
                objects.put(key, Class.forName(rb.getString(key)).newInstance());
            }
        }
        for (Injector i : injectors) {
            for (Object injectee : objects.values()) {
                if (injectee instanceof Injectable) {
                    i.inject((Injectable)injectee);
                }
            }
        }
    }
    
    /**
     * w肵IuWFNg擾B
     * YIuWFNg`̏ꍇɂnullԑB
     * @param name \[X̖
     */
    public Object getObject(String name) {
        return objects.get(name);
    }
    
    Injector createInjector(Class intf, String value) {
        Matcher m = NUMBER.matcher(value);
        if (m.matches()) {
            return new IntegerInjector(intf, value);
        }
        m = STRING.matcher(value);
        if (m.matches()) {
            return new StringInjector(intf, m.group(1));
        }
        return new ObjectInjector(intf, value);
    }
    
    /**
     * IuWFNgCWFNVB
     */
    class ObjectInjector implements Injector {

        Class targetInterface;
        String key;
        Method injector;

        /**
         * IuWFNgCWFN^[𐶐B
         * @param initTargetInterface CWFNgΏۂƂȂC^[tFCXB
         * @param initKey CWFNgΏۂ̃IuWFNgB
         */
        ObjectInjector(Class initTargetInterface, String initKey) {
            targetInterface = initTargetInterface;
            key = initKey;
            for (Method m : targetInterface.getMethods()) {
                if (m.getName().indexOf("inject") == 0 && m.getParameterTypes().length == 1) {
                    injector = m;
                    break;
                }
            }
        }

        public void inject(Injectable target) {
            assert injector != null : "no inject method";

            if (targetInterface.isInstance(target)) {
                invoke(injector, target, objects.get(key));
            }
        }
    }
}
