/*
 * MethodInterceptorTest.java
 * JUnit based test
 *
 * Created on 2005/10/06, 1:46
 */

package jp.co.gihyo.book.di.ch4;

import junit.framework.*;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 *
 * @author arton
 */
public class MethodInterceptorTest extends TestCase {
    
    public MethodInterceptorTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
    }

    protected void tearDown() throws Exception {
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(MethodInterceptorTest.class);
    }

    public interface TestInterface {
        void testMethod(int x, int y);
        int test2Method(String x);
        void testException();
    }
    
    public static class TestImpl implements TestInterface {
        public void testMethod(int x, int y) {
        }
        public int test2Method(String x) {
            return 2;
        }
        public void testException() {
            throw new IllegalArgumentException("test message");
        }
    }
    
    /**
     * NX jp.co.gihyo.book.di.ch3.MethodInterceptor  newProxy \bh̃eXg
     */
    String currentMethod;
    public void testNewProxy() {
        TestInterface ti = MethodInterceptor.weave(TestInterface.class, new TestImpl(), new MethodInterceptor.Interceptor() {
            public void beforeInvoke(Method m, Object[] args) {
                if (m.getName().equals("testException")) {
                    assertEquals(0, args.length);
                }
                assertEquals(currentMethod, m.getName());
            }
            public void afterInvoke(Method m, Object[] args, Object result) {
                assertEquals(currentMethod, m.getName());
            }
            public void exception(Method m, Object[] args, Throwable e) {
                assertEquals("test message", e.getMessage());
                assertEquals("testException", m.getName());
            }
        });
        currentMethod = "testMethod";
        ti.testMethod(1, 8);
        currentMethod = "test2Method";
        ti.test2Method("a");
        currentMethod = "testException";
        try {
            ti.testException();
            fail("no exception");
        } catch (Exception e) {
            assertTrue(e instanceof IllegalArgumentException);
        }
    }

}
